import React, {Component} from 'react';
import R from 'ramda';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import {connect} from 'react-redux';
import {ComponentNavigator} from '../playground/component-navigator/component-navigator';
import {XlReactDipReduxForm} from '../../xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form';
import FormKitchenTestComponent from '../playground/form-kitchen-test-component';
import {getHandlers, lookupSuggestedElements} from '../handlers/predefined-handlers';
import {getCollectionOfCiHandlers} from '../dip-page/dip-utils';
import formActions from '../../xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-actions';
import {formInitialState} from './form-initial-state';

import './form-page.less';

const mapStateToProps = (state) => state.forms;

const mapDispatchToProps = (dispatch) => ({
    initialize: ({formId, ...rest}) => dispatch(formActions.initialize({formId, ...rest})),
    mergeContext: (formId, context) => dispatch(formActions.mergeContext(formId, context))
});

@connect(mapStateToProps, mapDispatchToProps)
export class Form extends Component {
    static propTypes = {
        initialize: PropTypes.func,
        match: PropTypes.object,
        mergeContext: PropTypes.func
    };

    constructor(props) {
        super(props);

        const handlers = {
            CI: getHandlers(),
            LIST_OF_CI: getCollectionOfCiHandlers,
            LIST_OF_STRING: {nameComponentList: getHandlers()},
            LOOKUP_CI: getHandlers(lookupSuggestedElements),
            SET_OF_CI: getCollectionOfCiHandlers
        };

        const initializeForm = (formId) => {
            if (formId) {
                const {
                    metadata,
                    model,
                    objLabel,
                    validators
                } = formInitialState[formId];
                props.initialize({
                    formId,
                    handlers,
                    metadata,
                    model,
                    objLabel: objLabel || 'id',
                    validators
                });
            }
        };
        R.forEach(initializeForm, R.keys(formInitialState));
    }

    render() {
        const {match, mergeContext} = this.props;

        const xlReactDipForm = (formId) => <XlReactDipReduxForm
            autoCleanup={false}
            dateTimeFormat={'YYYY-MM-DDTHH:mm:ss.SSS'}
            focusOnFirstInvalidField={false}
            formId={formId}
        />;

        const xlReactDipFormWithWarnings = (formId) => <XlReactDipReduxForm
            autoCleanup={false}
            focusOnFirstInvalidField={false}
            formId={formId}
        />;

        const xlReactDipFormWithLookup = (formId) => <XlReactDipReduxForm
            autoCleanup={false}
            focusOnFirstInvalidField={false}
            formId={formId}
        />;

        const forms = {
            dynamicFormAsContainment: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-as-containment"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormAsContainment'}
                label={'Form with an asContainment property'}
                modelName={'dynamicFormAsContainment'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormAsContainment')}
            </FormKitchenTestComponent>,
            dynamicFormDefault: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-default"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormDefault'}
                label={'Form with default values'}
                modelName={'dynamicFormDefault'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormDefault')}
            </FormKitchenTestComponent>,
            dynamicFormEmpty: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-empty"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormEmpty'}
                label={'Empty form'}
                modelName={'dynamicFormEmpty'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormEmpty')}
            </FormKitchenTestComponent>,
            dynamicFormEmptyWithWarnings: <FormKitchenTestComponent
                className="xl-dip-dynamic-form-empty"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormEmptyWithWarnings'}
                label={'Empty form with warnings'}
                modelName={'dynamicFormEmptyWithWarnings'}
                updateState={mergeContext}>
                {xlReactDipFormWithWarnings('dynamicFormEmptyWithWarnings')}
            </FormKitchenTestComponent>,
            dynamicFormWithCategoryOptions: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-with-category-options"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormWithCategoryOptions'}
                label={'Form with category options'}
                modelName={'dynamicFormWithCategoryOptions'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormWithCategoryOptions')}
            </FormKitchenTestComponent>,
            dynamicFormWithMultipleCategories: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-with-multiple-categories"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormWithMultipleCategories'}
                label={'Form with multiple categories'}
                modelName={'dynamicFormWithMultipleCategories'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormWithMultipleCategories')}
            </FormKitchenTestComponent>,
            dynamicReadonlyForm: <FormKitchenTestComponent
                className="xl-react-dip-form-readonly"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicReadonlyForm'}
                label={'Readonly Form'}
                modelName={'dynamicReadonlyForm'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicReadonlyForm')}
            </FormKitchenTestComponent>,
            dynamicWithLookup: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-with-lookup-values"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicWithLookup'}
                label={'Form with lookup values'}
                modelName={'dynamicWithLookup'}
                updateState={mergeContext}>
                {xlReactDipFormWithLookup('dynamicWithLookup')}
            </FormKitchenTestComponent>
        };

        const isSingleItem = match && forms[match.params.name];

        const formStates = R.reduce((acc, key) => ({...acc, ...{[key]: this.props[key]}}), {}, R.keys(forms));

        return (
            <div className={classNames('form-page', {'single-item': isSingleItem})}>
                <ComponentNavigator componentType="form" state={formStates}/>
                <div className="page-content">
                    {isSingleItem ? forms[match.params.name] : R.values(forms)}
                </div>
            </div>
        );
    }
}
