import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {XlReactWidgetTable} from '../../xl-react-widget/xl-react-widget-table/xl-react-widget-table';
import {localeMessages} from '../../services/locale-service';
import ApiButton from './api-button';
import ElementNavTitle from './element-nav-title';
import ToggleComponent from './toggle-component';

export class TableTestComponent extends Component {
    static propTypes = {
        className: PropTypes.string,
        currentState: PropTypes.object.isRequired,
        modelName: PropTypes.string.isRequired,
        showBindings: PropTypes.bool,
        updateState: PropTypes.func.isRequired
    };

    constructor(props) {
        super(props);
        const {currentState, modelName} = this.props;
        const model = currentState[modelName];
        const isPagingDefined = RU.isDefined(model.paging);
        const dataStartIndex = isPagingDefined ? (model.paging.page - 1) * model.paging.resultsPerPage : 0;
        const dataEndIndex = isPagingDefined ? model.paging.resultsPerPage : model.data.length;
        this.state = {
            data: model.data.slice(dataStartIndex, dataEndIndex),
            filters: model.filters,
            paging: model.paging,
            selectedRowData: model.selectedRowData,
            selectRow: {
                classes: 'row-selected',
                clickToSelect: true,
                hideSelectColumn: true,
                mode: 'checkbox'
            },
            shouldShowFilters: model.shouldShowFilters,
            sorting: model.sorting,
            total: model.total
        };
    }

    handlePageChange = (page, resultsPerPage) => {
        this.setData({page, resultsPerPage});
    };

    handleFilterChange = (filters) => {
        this.setData({filters, page: 1});
    };

    handleSortChange = (name, order) => {
        this.setData({name, order});
    };

    handleRowDoubleClick = (row) => {
        this.setState({selectedRowData: row});
    };

    onToggleFilters = () => {
        if (this.state.shouldShowFilters) {
            this.tableApi.resetFilters();
        }

        this.setState({shouldShowFilters: !this.state.shouldShowFilters});
    };

    onToggleLoading = () => {
        const {modelName, updateState, currentState} = this.props;
        updateState(modelName, {
            ...currentState[modelName],
            loading: !currentState[modelName].loading
        });
    };

    onToggleSelectColumn = () => {
        const {selectRow} = this.state;
        this.setState({
            selectRow: {
                ...selectRow,
                hideSelectColumn: !selectRow.hideSelectColumn
            }
        });
    };

    setData = ({
        filters = this.state.filters,
        name = this.state.sorting.name,
        order = this.state.sorting.order,
        page = this.state.paging.page,
        resultsPerPage = this.state.paging.resultsPerPage
    }) => {
        const {currentState, modelName} = this.props;
        const model = currentState[modelName];
        const currentIndex = (page - 1) * resultsPerPage;

        const filtersByColumnName = R.pipe(
            R.toPairs,
            R.map(([colName, filterVal]) =>
                R.pipe(
                    R.prop(colName),
                    R.toLower,
                    R.compose(
                        R.includes,
                        R.toLower
                    )(filterVal)
                )
            )
        )(filters);

        const filter = R.filter(R.allPass(filtersByColumnName));

        const slice = R.slice(currentIndex, currentIndex + resultsPerPage);

        const sort = R.sortWith([order === 'asc' ? R.ascend(R.prop(name)) : R.descend(R.prop(name))]);

        const filteredData = filter(model.data);

        const data = R.pipe(
            sort,
            slice
        )(filteredData);

        this.setState({
            data,
            filters,
            paging: {
                ...this.state.paging,
                page,
                resultsPerPage
            },
            sorting: {name, order},
            total: filteredData.length
        });
    };

    buildClassName = (dataKey) => `xl-react-table-${dataKey}`;

    createColumnDefinition = (headerDef) => {
        const {currentState, modelName} = this.props;
        const {fieldFormatters} = currentState[modelName];
        const dataFormat = R.propOr(R.identity, headerDef.dataKey, fieldFormatters);

        return {
            classes: this.buildClassName(headerDef.dataKey),
            dataField: headerDef.dataKey,
            formatter: dataFormat,
            headerClasses: this.buildClassName(headerDef.dataKey),
            headerTitle: false,
            sort: true,
            text: headerDef.title
        };
    };

    registerTableApi = (api) => {
        this.tableApi = api;
    };

    render() {
        const {className, currentState, showBindings, modelName} = this.props;
        const model = currentState[modelName];
        const {
            error,
            fieldFormatters,
            keyField,
            layout,
            loading,
            loadingMessage,
            navigation: {
                description,
                title,
                type
            }
        } = model;

        const displayBindings = () =>
            showBindings ?
                <div className="testBindings">
                    <div>
                        <ToggleComponent
                            checked={this.state.shouldShowFilters}
                            clazzName="xl-react-widget-table-toggle-filters"
                            handleToggle={this.onToggleFilters}
                            label="Show filters"
                        />
                        <ToggleComponent
                            checked={!this.state.selectRow.hideSelectColumn}
                            clazzName="xl-react-widget-table-toggle-select"
                            handleToggle={this.onToggleSelectColumn}
                            label="Show multi select column"
                        />
                        <ToggleComponent
                            checked={model.loading}
                            clazzName="xl-react-widget-table-toggle-loading"
                            handleToggle={this.onToggleLoading}
                            label="Loading"
                        />
                        <span className="label">API</span>
                        <ApiButton
                            clazzName={'reset-table-filters-button'}
                            handleClick={() => this.tableApi.resetFilters()}
                            label="Reset filters"
                        />
                    </div>
                </div> :
                null;

        return (
            <div>
                <ElementNavTitle
                    description={description}
                    modelName={modelName}
                    title={title}
                    type={type}
                />
                <div className="xl-react-widget-table">
                    <div className="testComponent table-test-component">
                        <XlReactWidgetTable
                            className={className}
                            createColumnDefinition={this.createColumnDefinition}
                            data={this.state.data}
                            defaultErrorMessage={localeMessages.tableErrorMessage}
                            defaultNoDataText={localeMessages.tableEmptyDataMessage}
                            error={error}
                            fieldFormatters={fieldFormatters}
                            filter={this.state.shouldShowFilters ? this.state.filters : undefined}
                            keyField={keyField}
                            layout={layout}
                            loading={loading}
                            loadingMessage={loadingMessage}
                            onFilterChange={this.state.shouldShowFilters ? this.handleFilterChange : undefined}
                            onPageChange={this.handlePageChange}
                            onRowDoubleClick={this.handleRowDoubleClick}
                            onSortChange={this.handleSortChange}
                            paging={this.state.paging}
                            register={this.registerTableApi}
                            selectRow={this.state.selectRow}
                            sorting={this.state.sorting}
                            total={this.state.total}
                        />
                        {displayBindings()}
                    </div>
                </div>
            </div>
        );
    }
}
