import R from 'ramda';
import RS from 'ramdasauce';
import RU from '../utils/ramda-utils';
import {
    filterItems,
    invalidCiValidator,
    prepareViewModel,
    sortItems
} from '../xl-react-dip/xl-react-dip-collection-of-ci/xl-react-dip-collection-of-ci-service';
import {getInvalidDateValidators} from '../xl-react-widget/xl-react-widget-date/xl-react-widget-date';
import {getInvalidIntegerValidators} from '../xl-react-dip/xl-react-dip-integer/xl-react-dip-integer';
import {getValidators} from '../services/component-service';
import {validateComponent} from '../services/component-validate-service';
import {ERROR, WARNING} from '../utils/ci-utils';
import {createCiPropertyValidators} from './validator-utils';

const validationMessageValidator = (fieldName) => (model) => () => R.pipe(
    R.propOr([], 'validation-messages'),
    R.find(R.where({
        level: R.either(R.equals(WARNING), R.equals(ERROR)),
        property: R.equals(fieldName)
    })),
    R.cond([
        [R.isNil, R.always({})],
        [R.T, (prop) => ({
            condition: true,
            message: {
                show: false,
                text: prop.message,
                type: prop.level
            }
        })]
    ]))(model);

export const findCategoryNameByPropertyName = (metadata, propName) => R.pipe(
    R.find((val) => val.name === propName),
    (val) => R.isNil(val) ? val : val.category
)(metadata.properties);

const isGroupOpen = (groupProperties, collapsed) => {
    if (collapsed === 'auto') {
        const {required, hiddenRequired} = RU.countHits({
            hiddenRequired: R.whereEq({hidden: true, required: true}),
            required: R.propEq('required', true)
        })(groupProperties);
        return required - hiddenRequired > 0;
    }
    return !R.isNil(collapsed) ? collapsed : true;
};

export const getFieldNamesWithShownErrors = (invalidFieldNames, dirtyFieldNames, message) => {
    if (RU.isTrue(['show'], message)) {
        return invalidFieldNames;
    }
    return R.intersection(invalidFieldNames, dirtyFieldNames);
};

export const getInvalidCategoryNames = ({groupedProperties = [], pristinity, message, validity}) => {
    const invalidFieldNames = R.keys(R.filter(R.not, validity));
    const dirtyFieldNames = R.keys(R.filter(R.not, pristinity));
    const fieldNamesWithShownErrors = getFieldNamesWithShownErrors(invalidFieldNames, dirtyFieldNames, message);
    return R.reduce((acc, cat) => {
        const categoryPropNames = R.map(R.prop('name'), cat.properties);
        if (!R.isEmpty(R.intersection(fieldNamesWithShownErrors, categoryPropNames))) {
            return [...acc, cat.category];
        }
        return acc;
    }, [], groupedProperties);
};

export const groupByCategory = (dynamicFormProperties, collapsed, oldGroupedProperties, categoriesOptions = {}) => {
    const defaultToCommonGroup = R.map(R.over(R.lensProp('category'), R.defaultTo('Common')));
    const createGroupItem = R.map(([category, properties]) => ({
        category,
        categoryOptions: R.prop(category, categoriesOptions),
        isOpen: !RS.isNilOrEmpty(oldGroupedProperties) ?
            R.prop('isOpen', R.find(R.propEq('category', category), oldGroupedProperties)) :
            isGroupOpen(properties, collapsed),
        properties
    }));
    const sortByGroupName = R.sort(
        R.comparator((left, right) => left.category === 'Common' || left.category < right.category)
    );

    return R.pipe(
        defaultToCommonGroup,
        R.groupBy(R.prop('category')),
        R.toPairs,
        createGroupItem,
        sortByGroupName
    )(dynamicFormProperties);
};

export const prepareGroupedProperties = (metadata, collapsed, previousGroupedProperties) => {
    const isNonHidden = (property) => R.isNil(property.hidden) || property.hidden === false;

    const makeRequiredFalseIfAsContainment = (property) => {
        property.required = property.asContainment ? false : property.required;
        return property;
    };

    const getProperties = (obj) => obj.properties;

    return groupByCategory(
        R.pipe(getProperties, R.filter(isNonHidden), R.map(makeRequiredFalseIfAsContainment))(metadata),
        collapsed,
        previousGroupedProperties,
        metadata.categoryOptions
    );
};

const validateItems = async (mergeContext, model, objLabel, context, handlers, path) => {
    const items = prepareViewModel(model, objLabel);
    const validatedItems = await RU.invoke('validateItems', [R.map(R.prop('model'), items)], handlers) || items;
    const visibleItems = filterItems(validatedItems, context.searchTerm, objLabel);
    const newContext = {
        loadingItems: false,
        validatedItems,
        visibleItems
    };
    mergeContext(R.assocPath(path, newContext, {}));
    return newContext;
};

const defaultInitialContext = () => ({
    computeContext: () => ({}),
    getValidators: () => [],
    transformModel: R.identity
});

const collectionOfCiInitialContext = (sortAutomatically) =>
    ({handlers, model = [], objLabel, propName, contextPath = ['context', propName]}) => {
        const transformModel = (val) => sortAutomatically ? sortItems(val, objLabel) : val;
        const items = prepareViewModel(transformModel(model), objLabel);
        return {
            ...defaultInitialContext({}),
            computeContext: (mergeContext, oldContext, newModel) => {
                mergeContext(R.assocPath(contextPath, {loadingItems: true}, {}));
                return validateItems(mergeContext, newModel, objLabel, oldContext, handlers, contextPath);
            },
            getValidators: (context) => [invalidCiValidator(context)],
            searchTerm: '',
            transformModel,
            validatedItems: items,
            visibleItems: items
        };
    };

const listOfStringInitialContext = (args) => ({
    ...defaultInitialContext(args),
    typed: ''
});

const integerInitialContext = () => ({
    ...defaultInitialContext({}),
    getValidators: () => getInvalidIntegerValidators()
});

const datetimeInitialContext = () => ({
    ...defaultInitialContext({}),
    getValidators: () => getInvalidDateValidators()
});

const initialContext = {
    DATE: datetimeInitialContext,
    INTEGER: integerInitialContext,
    LIST_OF_CI: collectionOfCiInitialContext(false),
    LIST_OF_STRING: listOfStringInitialContext,
    SET_OF_CI: collectionOfCiInitialContext(true)
};

export const getInitialContext = ({contextPath, handlers = {}, ignoredProperties = [], metadata, objLabel}, model) =>
    R.reduce((acc, propMetadata) => {
        const args = {
            ...propMetadata,
            contextPath,
            handlers: handlers[propMetadata.kind],
            model: model[propMetadata.name],
            objLabel,
            propName: propMetadata.name
        };
        return R.contains(propMetadata.name, ignoredProperties) ? acc : {
            ...acc,
            [propMetadata.name]: RU.invoke(propMetadata.kind, [args], initialContext) || defaultInitialContext(args)
        };
    }, {}, metadata.properties);

export const validateField = (props, propName, context, pristine, value) => {
    const metadata = R.find(R.propEq('name', propName), props.metadata.properties) || {};
    const getAllValidators = () => getValidators(
        {...props, metadata: {...metadata, ...props.metadata}},
        [
            validationMessageValidator(propName)(props.model),
            ...RU.invoke('getValidators', [context], context) || [],
            ...createCiPropertyValidators(props, metadata)]
    );
    return validateComponent(props, getAllValidators(), value, pristine);
};

const validateFields = (props, path = [], initValue = {}) => R.reduce((acc, key) => {
    const {message, valid} = validateField(
        props,
        key,
        props.context[key],
        props.pristinity[key],
        R.path(['model', ...path, key], props)
    );
    return {
        messages: {
            ...acc.messages,
            [key]: message
        },
        validity: {
            ...acc.validity,
            [key]: valid
        }
    };
}, initValue);

export const validateAllFields = (props) => {
    const modelValidation = R.pipe(
        R.omit(['external-properties', 'validation-messages']),
        R.keys,
        validateFields(props)
    )(props.model);
    return R.pipe(
        R.keys,
        validateFields(props, ['external-properties'], modelValidation)
    )(props.model['external-properties'] || {});
};
