import classNames from 'classnames';
import * as KeyCodes from 'keycode-js';
import PropTypes from 'prop-types';
import R from 'ramda';
import React, {PureComponent} from 'react';
import TagsInput from 'react-tagsinput';
import 'react-tagsinput/react-tagsinput.css';
import {localeMessages} from '../../services/locale-service';
import {validationMessageClasses} from '../../utils/css-utils';
import RU from '../../utils/ramda-utils';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';
import {XlReactWidgetAutocomplete} from '../xl-react-widget-autocomplete/xl-react-widget-autocomplete';
import {tagsSuggestionBehaviorPropsType} from './suggestion-behavior/tags-suggestion-behavior-type';
import {inputTypes} from './xl-react-widget-tags-service';
import {XlReactWidgetTagsModelProps} from './xl-react-widget-tags-model-props';
import {getTagsField, getRemainingTags} from './xl-react-widget-tags-dom-manipulation';

import './xl-react-widget-tags.less';

export class XlReactWidgetTagsComponent extends PureComponent {
    static propTypes = {
        addTagOnEnter: PropTypes.func,
        allowAddOnlyUniqueSuggestion: PropTypes.bool,
        assignRef: PropTypes.func,
        autocompleteMetadata: PropTypes.object,
        className: PropTypes.string,
        description: PropTypes.string,
        disabled: PropTypes.bool,
        handleAutocompleteSelectAll: PropTypes.func,
        handleModelChange: PropTypes.func,
        handlers: PropTypes.object,
        handleSuggestionSelection: PropTypes.func,
        handleTextInputChange: PropTypes.func,
        inputType: PropTypes.oneOf(R.values(inputTypes)),
        label: PropTypes.string,
        message: xlMessageTypeProp,
        model: XlReactWidgetTagsModelProps,
        objLabel: PropTypes.string,
        onValidityChange: PropTypes.func,
        registerChildApi: PropTypes.func,
        remainingTags: XlReactWidgetTagsModelProps,
        removeKeys: PropTypes.arrayOf(PropTypes.number),
        required: PropTypes.bool,
        requiredDirection: PropTypes.string,
        showNumberOfRemainingTags: PropTypes.bool,
        subActions: PropTypes.arrayOf(PropTypes.shape({
            action: PropTypes.func.isRequired,
            isDisabled: PropTypes.func.isRequired,
            label: PropTypes.string.isRequired
        })),
        suggestionBehavior: tagsSuggestionBehaviorPropsType,
        tag: PropTypes.string,
        tagTransform: PropTypes.func.isRequired,
        valid: PropTypes.bool
    };

    static defaultProps = {tagTransform: (val) => val};

    constructor(props) {
        super(props);
        this.state = {collapseTags: true};
    }

    componentDidMount() {
        this.tagsField = getTagsField(this.inputField, this.props.model);
    }

    componentDidUpdate() {
        this.tagsField = getTagsField(this.inputField, this.props.model);
    }

    handleAddTagOnEnter = (event) => {
        this.props.addTagOnEnter(event, this.autocompleteApi);
    };

    handleMouseLeave = () => {
        if (this.props.disabled) {
            this.collapseTags();
        }
    };

    handleMoreLabelClick = () => {
        this.expandTags();
        if (this.addTagInputField) {
            this.addTagInputField.focus();
        } else {
            this.handleLayoutFocus();
        }
    };

    handleLayoutFocus = () => {
        RU.invoke('focus', [], this.autocompleteApi);
    };

    handleOnShowAllClick = () => this.isAutocomplete() && this.inputField && this.inputField.focus();

    collapseTags = () => {
        this.setState({collapseTags: true}, () => {
            this.tagsField.scrollTop(0);
        });
    };

    expandTags = () => {
        this.setState({collapseTags: false});
    };

    defaultRenderLayout = (tagComponents, inputComponent) => {
        const isToShowScrollableTagsLayout = this.isAutocomplete() && !R.isEmpty(this.props.model);
        return <span>
            <span
                className={classNames({'scrollable-tags-layout': isToShowScrollableTagsLayout})}
                onClick={this.handleLayoutFocus}>
                {tagComponents}
            </span>
            {inputComponent}
        </span>;
    };

    assignAddTagInputField = (field) => {
        if (field) {
            this.addTagInputField = R.head(field.children);
        }
    };

    assignRef = (inputField) => {
        this.inputField = inputField;
        this.props.assignRef(inputField);
    };

    isAutocomplete = () => this.props.inputType === inputTypes.AUTOCOMPLETE;

    registerAutocompleteApi = (api) => {
        this.autocompleteApi = api;
        this.props.registerChildApi(api);
    };

    renderInputTextbox = (tagsProps) => {
        const {addTag, ...others} = tagsProps;
        const {tag} = this.props;

        const handleButtonOnClick = () => {
            addTag(this.addTagInputField.value);
        };

        const addButton = !R.isEmpty(tag) &&
            <button className="xl-btn-add" onMouseDown={handleButtonOnClick} type="button">
                {localeMessages.addButton}
            </button>;

        const inputProps = R.evolve({
            onBlur: (currentOnBlurHandler) => (event) => {
                this.collapseTags();
                currentOnBlurHandler();
                addTag(event.target.value);
            },
            onFocus: (currentOnFocusHandler) => () => {
                this.expandTags();
                currentOnFocusHandler();
            }
        }, others);

        return <span ref={this.assignAddTagInputField}>
            <input {...({type: 'text', value: tag, ...inputProps})}/>
            {addButton}
        </span>;
    };

    renderAutocomplete = (tagsProps) => {
        const others = R.omit(['addTag', 'disabled'], tagsProps);
        const {
            autocompleteMetadata,
            handlers,
            handleSuggestionSelection,
            objLabel,
            subActions,
            suggestionBehavior,
            tag
        } = this.props;

        const inputProps = R.evolve({
            onBlur: (currentOnBlurHandler) => () => {
                this.collapseTags();
                currentOnBlurHandler();
                this.props.handleTextInputChange({target: {value: ''}});
            },
            onFocus: (currentOnFocusHandler) => () => {
                this.expandTags();
                currentOnFocusHandler();
            },
            onKeyDown: (currentOnKeyDownHandler) => (event) =>
                R.includes(event.keyCode, [KeyCodes.KEY_RETURN, KeyCodes.KEY_TAB]) ?
                    this.handleAddTagOnEnter(event) : currentOnKeyDownHandler(event)
        }, others);

        return <XlReactWidgetAutocomplete
            clearInputOnBlur={true}
            disabled={tagsProps.disabled}
            handlers={handlers}
            inputProps={inputProps}
            metadata={autocompleteMetadata}
            model={tag}
            objLabel={objLabel}
            onSuggestionSelected={handleSuggestionSelection}
            register={this.registerAutocompleteApi}
            subActions={subActions}
            suggestionBehavior={suggestionBehavior}
        />;
    };

    renderInput = this.isAutocomplete() ? this.renderAutocomplete : this.renderInputTextbox;

    render() {
        const {
            className, disabled, message, handleModelChange,
            handleTextInputChange, model, showNumberOfRemainingTags, suggestionBehavior, tag, valid
        } = this.props;

        const remainingTags = getRemainingTags(this.tagsField, model);

        const showNoOfRemainingTags = () =>
            showNumberOfRemainingTags &&
            this.state.collapseTags &&
            remainingTags.length > 0;

        const inputClasses = classNames(
            className,
            {'collapse-tags': showNoOfRemainingTags()},
            validationMessageClasses(valid, message)
        );

        const showAll = () => this.isAutocomplete() && this.inputField &&
            <i className="xl-icon arrow-down-icon show-all" onClick={this.handleOnShowAllClick}/>;

        const showSearchIcon = () => this.isAutocomplete() && <i className="xl-icon search-icon"/>;

        const renderCustomPlaceholder = () => this.state.collapseTags && R.isEmpty(model) &&
            <span className="custom-placeholder" onClick={this.handleOnShowAllClick}>
                {this.props.autocompleteMetadata.placeholder}
            </span>;

        return <XlReactFormItemControl {...({
            ...this.props,
            controlClass: 'xl-react-widget-tags',
            message
        })}>
            <div className={inputClasses} onMouseLeave={this.handleMouseLeave}>
                <span className="react-tagsinput-wrapper">
                    {showSearchIcon()}
                    {renderCustomPlaceholder()}
                    <TagsInput
                        addOnBlur={false}
                        disabled={disabled}
                        focusedClassName={disabled ? '' : 'react-tagsinput--focused'}
                        inputProps={{onChange: handleTextInputChange, placeholder: ''}}
                        inputValue={tag}
                        onChange={handleModelChange}
                        ref={this.assignRef}
                        renderInput={this.renderInput}
                        renderLayout={this.defaultRenderLayout}
                        renderTag={suggestionBehavior.renderTag(this)}
                        value={model}
                    />
                    {
                        showNoOfRemainingTags() &&
                        <div
                            className="remaining-tags"
                            onClick={this.handleMoreLabelClick}>
                            <span>
                                {remainingTags.length} {localeMessages.moreLabel}
                            </span>
                        </div>
                    }
                    {showAll()}
                </span>
            </div>
        </XlReactFormItemControl>;
    }
}
