import R from 'ramda';
import {
    Action,
    ActionUtil,
    Expectation,
    WaitCondition
} from 'protractor-base-dsl';
import json from 'format-json';
import XlComponentsHelper from '../../common/xl-components-helper';
import {createFunctions} from '../dip/dip-collection-of-ci';
import {
    getLookupDescription,
    getLookupKeyField,
    getLookupProviderField,
    getToggleButton,
    getLookupInputField,
    getLookupTextareaField
} from './lookup-field';

const componentSelector = '.testComponent';

const expectFieldsEqualsTo = (selector, fieldNames, value) =>
    Expectation.textEquals(selector, json.plain(R.reduce((acc, fieldName) => ({
        ...acc,
        [fieldName]: value
    }), {}, fieldNames)));

const expectAllFieldsEqualsTo = (selector, value) =>
    expectFieldsEqualsTo(selector, ['nameComponentString', 'nameComponentStringWithTypeHint',
        'nameComponentStringLarge', 'nameComponentStringMonaco', 'nameComponentInteger', 'nameComponentBoolean',
        'nameComponentSet', 'nameComponentList', 'nameComponentCi', 'nameStringEnum', 'nameRadioEnum',
        'mapStringStringName', 'nameSetOfCi', 'nameListOfCi', 'datetimeName', 'datetimeStringName'], value);

const getElementInForm = (fieldLabel) =>
    XlComponentsHelper.getElementByLabel(fieldLabel, element(By.css(componentSelector)));

const expectFieldToBe = (fieldLabel, elementSelector, attrName, value) =>
    Expectation.attributeContainsValue(getElementInForm(fieldLabel).element(By.css(elementSelector)), 'value', value);

const expectCollectionOfCiFieldHasInvalidOption = (fieldLabel, itemNumber) => {
    const element = getElementInForm(fieldLabel);
    createFunctions(element).expectInvalidItem(itemNumber);
};

const expectCollectionOfCiFieldHasValidOption = (fieldLabel, itemNumber) => {
    const element = getElementInForm(fieldLabel);
    createFunctions(element).expectValidItem(itemNumber);
};

const expectFieldHasAttribute = (fieldLabel, elementSelector, attrName) =>
    Expectation.hasAttribute(getElementInForm(fieldLabel).element(By.css(elementSelector)), attrName);

const doIfPresent = (fieldLabel, elementSelector, func) =>
    getElementInForm(fieldLabel).element(By.css(elementSelector)).isPresent()
        .then((present) => {
            if (present) {
                func();
            }
        });

const expectDipMapHasScrollableContainer = (fieldLabel) =>
    Expectation.displayed(getElementInForm(fieldLabel)
        .element(By.css('.xl-react-components-input .table-scrollable-container')));

const expectFormElementTextToEqual = (fieldLabel, elementSelector, text) =>
    Expectation.textEquals(getElementInForm(fieldLabel).element(By.css(elementSelector)), text);

export const expectFieldPresent = (fieldLabel, elementSelector) =>
    Expectation.displayed(getElementInForm(fieldLabel).element(By.css(elementSelector)));

export const expectFieldNotPresent = (fieldLabel, elementSelector) =>
    Expectation.notPresent(getElementInForm(fieldLabel).element(By.css(elementSelector)));

export const expectFieldHasValue = (fieldLabel, elementSelector, value) =>
    expectFieldToBe(fieldLabel, elementSelector, 'value', value);

export const expectLookupFieldHasValue = (fieldLabel, elementSelector, value) => {
    Expectation.displayed(getToggleButton(fieldLabel));
    expectFieldToBe(fieldLabel, elementSelector, 'value', value);
};

export const expectLookupHasWarning = (fieldLabel, message) => {
    Expectation.textEquals(getLookupDescription(fieldLabel, '.warning'), message);
};

export const expectToggledInputLookupHasValues = (fieldLabel, provider, key) => {
    Expectation.displayed(getToggleButton(fieldLabel));
    Expectation.textEquals(getLookupProviderField(fieldLabel), provider);
    Expectation.textEquals(getLookupKeyField(fieldLabel), key);
};

export const expectFieldHasText = (fieldLabel, elementSelector, value) =>
    expectFieldToBe(fieldLabel, elementSelector, 'text', value);

export const expectInputFieldChecked = (fieldLabel) =>
    expectFieldHasAttribute(fieldLabel, 'input[type="checkbox"]', 'checked');

export const expectFieldDisabled = (fieldLabel) =>
    expectFieldHasAttribute(fieldLabel, 'input', 'disabled');

export const expectFieldReadonly = (elementSelector, fieldLabel) =>
    expectFieldHasAttribute(fieldLabel, elementSelector, 'readonly');

export const expectMonacoFieldReadonly = (fieldLabel) =>
    Expectation.attributeContainsValue(getElementInForm(fieldLabel)
        .element(By.css('.xl-react-component-input-wrapper')), 'class', 'readonly');

export const expectFirstLineMonacoFieldHasText = (fieldLabel, text) => {
    const firstLine = getElementInForm(fieldLabel)
        .element(By.css('.view-lines .view-line:nth-child(1) span span'));
    Expectation.textEquals(firstLine, text);
};

export const expectFieldInvalid = (fieldLabel) =>
    Expectation.hasClassName(getElementInForm(fieldLabel), 'invalid');

export const expectAllFieldsArePristine = () =>
    expectAllFieldsEqualsTo('.testBindings .pristine.value', true);

export const expectFieldsArePristine = (fieldNames) =>
    expectFieldsEqualsTo('.testBindings .pristine.value', fieldNames, true);

export const expectFieldsAreValid = (fieldNames) =>
    expectFieldsEqualsTo('.testBindings .valid.value', fieldNames, true);

export const expectFieldsAreInvalid = (fieldNames) =>
    expectFieldsEqualsTo('.testBindings .valid.value', fieldNames, false);

export const expectedFieldsPristinity = (expected) =>
    Expectation.textEquals('.testBindings .pristine.value', json.plain(expected));

export const expectAllFieldsAreValid = () =>
    expectAllFieldsEqualsTo('.testBindings .valid.value', true);

export const expectAllNotEmptyFieldsAreInvalid = () =>
    Expectation.textEquals('.testBindings .valid.value', json.plain({
        nameComponentString: false,
        nameComponentStringWithTypeHint: false,
        nameComponentStringLarge: false,
        nameComponentStringMonaco: false,
        nameComponentInteger: false,
        nameComponentBoolean: true,
        nameComponentSet: false,
        nameComponentList: false,
        nameComponentCi: false,
        nameStringEnum: false,
        nameRadioEnum: false,
        mapStringStringName: false,
        nameSetOfCi: false,
        nameListOfCi: false,
        datetimeName: true,
        datetimeStringName: true
    }));

export const expectDescription = (fieldLabel, message) =>
    expectFormElementTextToEqual(fieldLabel, '.xl-react-components-description', message);

export const expectAllFieldsHaveDescription = () => {
    expectDescription('Label component String',
        'Description component String (Property: nameComponentString)');
    expectDescription('Label component StringWithTypeHint',
        'Description component String (Property: nameComponentStringWithTypeHint)');
    expectDescription('Label component StringLarge',
        'Description component StringLarge (Property: nameComponentStringLarge)');
    expectDescription('Label component Integer',
        'Description component Integer (Property: nameComponentInteger)');
    expectDescription('Label component Boolean',
        'Description component Boolean (Property: nameComponentBoolean)');
    expectDescription('Label component Set',
        'Description component Set (Property: nameComponentSet)');
    expectDescription('Label component List',
        'Description component List (Property: nameComponentList)');
    expectDescription('Label component CI',
        'Description component CI (Property: nameComponentCi)');
    expectDescription('Label String enum',
        'Description enum of string (Property: nameStringEnum)');
    expectDescription('Label Radio enum',
        'Description enum of strings as radio (Property: nameRadioEnum)');
    expectDescription('Label component Map String String',
        'Description component Map String String (Property: mapStringStringName)');
    expectDescription('Label component Set Of CI',
        'Description component Set Of CI (Property: nameSetOfCi)');
    expectDescription('Label component List Of CI',
        'Description component List Of CI (Property: nameListOfCi)');
    expectDescription('Label component Date Time',
        'Description component Date Time (Property: datetimeName)');
};

export const expectModelToEqual = (model) =>
    Expectation.textEquals('.testBindings .model.value', json.plain(model));

export const expectPlainModelToEqual = (model) =>
    Expectation.textEquals('.testBindings .model.value', model);

export const expectDipSetOfStringNoTags = (fieldLabel) =>
    expectFieldNotPresent(fieldLabel, '.react-tagsinput .react-tagsinput-tag');

export const expectDipSetOfStringsHasTags = (fieldLabel, tags) => {
    for (let tagIndex = 0; tagIndex < tags.length; tagIndex += 1) {
        const rowCss = '.xl-virtualized-table-body .xl-virtualized-table-row';
        const tagSelector = `${rowCss}:nth-child(${tagIndex + 1}) input`;
        expectFieldHasValue(fieldLabel, tagSelector, tags[tagIndex]);
    }
};

export const expectDipCollectionHasValues = (fieldLabel, values) => {
    for (let valueInd = 0; valueInd < values.length; valueInd += 1) {
        const itemSelector = `.xl-scrollable-panel .xl-list-row:nth-child(${valueInd + 1}) .text-container`;
        expectFormElementTextToEqual(fieldLabel, itemSelector, values[valueInd]);
    }
};

export const expectCategoryTitleHidden = (categoryName) =>
    Expectation.notDisplayed(XlComponentsHelper.getCategoryByLabel(categoryName).element(By.css('.xl-react-category')));

export const expectCategoryNotCollapsible = (categoryName) => {
    const categoryPanel = XlComponentsHelper.getCategoryByLabel(categoryName);
    const categoryContent = categoryPanel.element(By.css('.xl-react-category-content'));
    const category = categoryPanel.element(By.css('.xl-react-category'));
    Expectation.displayed(categoryContent);
    Action.click(category);
    return Expectation.displayed(categoryContent);
};

export const expectValidCategory = (categoryNumber) => {
    Expectation.displayed(`.open:nth-child(${categoryNumber}) .xl-react-category`);
    const selector = `${componentSelector} .open:nth-child(${categoryNumber}) .xl-react-category.invalid-category`;
    return Expectation.notPresent(selector);
};

export const expectInvalidCategory = (categoryNumber) => {
    const selector = `${componentSelector} .open:nth-child(${categoryNumber}) .xl-react-category.invalid-category`;
    return Expectation.displayed(selector);
};

export const expectDipCollectionHasNoValues = (fieldLabel) =>
    expectFieldNotPresent(fieldLabel, '.xl-scrollable-panel .xl-list-row');

const getRowSelector = (cellNumber) => `.xl-virtualized-table-row:nth-child(${cellNumber})`;

const expectMapEntry = (fieldLabel, cellNumber, key, value, valueSelector) => {
    const rowSelector = getRowSelector(cellNumber);
    const checkIt = () => expectDipMapHasScrollableContainer(fieldLabel);
    doIfPresent(fieldLabel, '.xl-virtualized-table-row', checkIt);
    expectFieldHasValue(fieldLabel, `${rowSelector} .key-div input`, key);
    return expectFieldHasValue(fieldLabel, valueSelector, value);
};

export const expectDipMapHasEntry = (fieldLabel, cellNumber, key, value) =>
    expectMapEntry(fieldLabel, cellNumber, key, value,
        `${getRowSelector(cellNumber)} .value-div textarea`);

export const expectEncryptedMapHasEntry = (fieldLabel, cellNumber, key, value) =>
    expectMapEntry(fieldLabel, cellNumber, key, value,
        `${getRowSelector(cellNumber)} .value-div input[type="password"]`);

export const expectDipMapIsEmpty = (fieldLabel) =>
    Expectation.notPresent(getElementInForm(fieldLabel).element(By.css('.table-wrapper')));

export const expectDateTimeToEqual = (fieldLabel, dateValue, timeValue) => {
    expectFieldHasValue(fieldLabel, '.xl-react-widget-date input', dateValue);
    return expectFieldHasValue(fieldLabel, '.xl-react-widget-time input', timeValue);
};

export const expectDipMapDisabled = (fieldLabel) => {
    expectFieldHasAttribute(fieldLabel, '.add-row-button', 'disabled');

    const checkEntries = () => {
        expectDipMapHasScrollableContainer(fieldLabel);
        expectFieldNotPresent(fieldLabel, '.xl-virtualized-table-row:nth-child(1) .close-icon');

        const keySelector = '.xl-virtualized-table-row:nth-child(1) .key-div input';
        expectFieldHasAttribute(fieldLabel, keySelector, 'readonly');

        const valueSelector = '.xl-virtualized-table-row:nth-child(1) .value-div textarea';
        expectFieldHasAttribute(fieldLabel, valueSelector, 'readonly');
    };

    return doIfPresent(fieldLabel, '.xl-virtualized-table-row', checkEntries);
};

export const expectDipStringsDisabled = (fieldLabel) => {
    expectFieldHasAttribute(fieldLabel, '.add-row-button', 'disabled');

    const tagSelector = '.xl-virtualized-table-body .xl-virtualized-table-row:nth-child(1) input';
    expectFieldHasAttribute(fieldLabel, tagSelector, 'readonly');
};

export const cleanValue = (fieldLabel) => {
    const action = () =>
        Action.click(getElementInForm(fieldLabel).element(By.css('.close-icon')));
    const condition = () =>
        WaitCondition.notPresent(getElementInForm(fieldLabel).element(By.css('.close-icon')));
    return ActionUtil.repeatAction(action, condition);
};

export const expectErrorMessage = (fieldLabel, errorMessage) =>
    expectFormElementTextToEqual(fieldLabel, '.xl-react-components-description.error', errorMessage);

export const expectWarningBorder = (fieldLabel) =>
    expectFieldPresent(fieldLabel, '.valid-with-warnings');

export const expectWarningMessage = (fieldLabel, message) =>
    expectFormElementTextToEqual(fieldLabel, '.xl-react-components-description.warning', message);

export const checkDescriptionAndErrorMessage = (fieldLabel, message, cleanValueFunctions) => {
    expectDescription(fieldLabel, message);
    if (!cleanValueFunctions) {
        cleanValue(fieldLabel);
    } else {
        for (const cleanValueFunction of cleanValueFunctions) {
            cleanValueFunction(fieldLabel);
        }
    }
    expectErrorMessage(fieldLabel, 'This field is required');
};

const removeItemOrValueInElement = (fieldLabel, elementSelector) =>
    Action.click(getElementInForm(fieldLabel).element(By.css(elementSelector)));

export const removeFirstTag = (fieldLabel) => {
    const selector = '.xl-virtualized-table-body .xl-virtualized-table-row:nth-child(1) .close-icon';
    return removeItemOrValueInElement(fieldLabel, selector);
};

export const removeCollectionItemByIndex = (fieldLabel, index) => {
    const elementSelector = `.xl-scrollable-panel .xl-list-row:nth-child(${index}) .close-icon`;
    return removeItemOrValueInElement(fieldLabel, elementSelector);
};

export const removeFirstCollectionItem = (fieldLabel) => {
    const elementSelector = '.xl-scrollable-panel .xl-list-row:nth-child(1) .close-icon';
    return removeItemOrValueInElement(fieldLabel, elementSelector);
};

export const removeFirstMapEntry = (fieldLabel) => {
    const elementSelector = '.xl-virtualized-table-row:nth-child(1) .close-icon';
    return removeItemOrValueInElement(fieldLabel, elementSelector);
};

const typeInField = (fieldElement, text, append) => {
    Action.click(fieldElement);
    browser.sleep(500);
    if (append) {
        Action.typeText(fieldElement, text);
    } else {
        Action.typeNewText(fieldElement, text);
    }
};

export const setInputValue = (fieldLabel, text) => {
    const inputElement = getElementInForm(fieldLabel).element(By.css('input'));
    typeInField(inputElement, text, true);
};

export const setRadioValue = (fieldLabel, option) => {
    const inputElement = getElementInForm(fieldLabel)
        .element(By.css(`.xl-react-widget-radio:nth-child(${option}) .radio-group-option`));
    Action.click(inputElement);
};

export const setTextAreaValue = (fieldLabel, value) => {
    const elementSelector = 'textarea';
    return Action.typeText(getElementInForm(fieldLabel).element(By.css(elementSelector)), value);
};

export const addTag = (fieldLabel, value) => {
    const elementSelector = '.testComponent .react-tagsinput-input';
    Action.typeText(getElementInForm(fieldLabel).element(By.css(elementSelector)), value);
    return Action.click(getElementInForm(fieldLabel).element(By.css('.react-tagsinput button')));
};

export const removeTag = (fieldLabel) =>
    Action.click(getElementInForm(fieldLabel).element(By.css('.react-tagsinput-remove')));

export const clickOnFieldLabel = (fieldLabel) =>
    Action.click(getElementInForm(fieldLabel).element(By.css('.xl-react-components-label label')));

const toggleLookup = (fieldLabel) => {
    Action.click(getElementInForm(fieldLabel).element(By.css('.xl-react-widget-switch-button')));
};

export const typeTextInLookupField = (fieldLabel, text, isTextarea) => {
    const element = isTextarea ? getLookupTextareaField(fieldLabel) : getLookupInputField(fieldLabel);
    typeInField(element, text);
};

export const typeTextInToggledLookupField = (fieldLabel, providerText, keyText) => {
    typeInField(getLookupKeyField(fieldLabel), keyText);
    typeInField(getLookupProviderField(fieldLabel), providerText);
};

const Form = {
    addTag,
    checkDescriptionAndErrorMessage,
    cleanValue,
    clickOnFieldLabel,
    expectCategoryTitleHidden,
    expectCategoryNotCollapsible,
    expectEncryptedMapHasEntry,
    expectFieldPresent,
    expectFieldNotPresent,
    expectFieldHasValue,
    expectFieldHasText,
    expectedFieldsPristinity,
    expectInputFieldChecked,
    expectFieldDisabled,
    expectFieldReadonly,
    expectFieldInvalid,
    expectFirstLineMonacoFieldHasText,
    expectMonacoFieldReadonly,
    expectAllFieldsArePristine,
    expectAllFieldsAreValid,
    expectAllNotEmptyFieldsAreInvalid,
    expectCollectionOfCiFieldHasInvalidOption,
    expectCollectionOfCiFieldHasValidOption,
    expectAllFieldsHaveDescription,
    expectLookupFieldHasValue,
    expectModelToEqual,
    expectPlainModelToEqual,
    expectDipSetOfStringNoTags,
    expectDipSetOfStringsHasTags,
    expectDipCollectionHasValues,
    expectValidCategory,
    expectInvalidCategory,
    expectDipCollectionHasNoValues,
    expectDipMapHasEntry,
    expectDipMapIsEmpty,
    expectDateTimeToEqual,
    expectDipMapDisabled,
    expectDipStringsDisabled,
    expectDescription,
    expectErrorMessage,
    expectLookupHasWarning,
    expectToggledInputLookupHasValues,
    expectFieldsArePristine,
    expectFieldsAreValid,
    expectFieldsAreInvalid,
    expectWarningBorder,
    expectWarningMessage,
    getElementInForm,
    removeCollectionItemByIndex,
    removeFirstCollectionItem,
    removeFirstMapEntry,
    removeFirstTag,
    removeItemOrValueInElement,
    removeTag,
    setInputValue,
    setRadioValue,
    setTextAreaValue,
    typeTextInLookupField,
    typeTextInToggledLookupField,
    toggleLookup
};

export default Form;
global.Form = Form;
