import React from 'react';

import Validations from '../../validations-spec-helper';
import {ERROR, WARNING} from '~/app/utils/ci-utils';
import {XlReactWidgetString} from '~/app/xl-react-widget/xl-react-widget-string/xl-react-widget-string';

describe('XlReactWidgetString', () => {
    const validations = new Validations('.xl-react-widget-string');

    it('should have label', () => {
        const {wrapper} = validations.buildFrom(<XlReactWidgetString metadata={{label: 'My Label'}}/>);
        expect(wrapper.find('.xl-react-components-input')).toExist();
        expect(wrapper.find('.xl-react-components-input-full')).not.toExist();
        expect(wrapper.find('.xl-react-components-label label')).toHaveText('My Label');
    });

    it('should render icon in input field', () => {
        const inputIconRenderer = () => <i/>;
        const {wrapper} = validations.buildFrom(<XlReactWidgetString inputIconRenderer={inputIconRenderer}/>);
        expect(wrapper.find('.xl-react-components-input-full')).toExist();
        expect(wrapper.find('.xl-react-components-input-full i')).toExist();
    });

    it('should have label with start (*) when component is required', () => {
        const {wrapper} = validations.buildFrom(<XlReactWidgetString metadata={{label: 'My Label', required: true}}/>);
        expect(wrapper.find('.xl-react-components-input')).toExist();
        expect(wrapper.find('.xl-react-components-input-full')).not.toExist();
        expect(wrapper.find('.xl-react-components-label label')).toHaveText('My Label*');
    });

    it('should show description when it specified in options', () => {
        const {description} = validations.buildFrom(
            <XlReactWidgetString metadata={{description: 'This is a string component'}}/>
        );
        expect(description()).toExist();
        expect(description()).toHaveText('This is a string component');
    });

    it('should not have "clear" button when component is empty', () => {
        const {clearButton, inputField} = validations.buildFrom(<XlReactWidgetString/>);
        expect(inputField()).toHaveValue('');
        expect(clearButton()).not.toExist();
    });

    it('should have "clear" button when input contains some data', () => {
        const {clearButton, inputField} = validations.buildFrom(<XlReactWidgetString model="someText"/>);
        expect(inputField()).toHaveValue('someText');
        expect(clearButton()).toExist();
    });

    it('check that placeholder is applied', () => {
        const {clearButton, inputField} =
            validations.buildFrom(<XlReactWidgetString metadata={{placeholder: 'my placeholder'}}/>);
        expect(inputField()).toHaveValue('');
        expect(inputField()).toHaveProp('placeholder', 'my placeholder');
        expect(clearButton()).not.toExist();
    });

    it('should show disabled input when "disabled" attribute is set to "true"', () => {
        const {clearButton, inputField} = validations
            .buildFrom(<XlReactWidgetString disabled={true} model="someText"/>);

        expect(inputField()).toHaveValue('someText');
        expect(inputField().props().readOnly).toEqual(true);
        expect(clearButton()).not.toExist();
    });

    it('should show disabled input when "displayMode" attribute is set to "visual"', () => {
        const {clearButton, inputField} =
            validations.buildFrom(<XlReactWidgetString displayMode="visual" model="someText"/>);

        expect(inputField()).toHaveValue('someText');
        expect(inputField().props().readOnly).toEqual(true);
        expect(clearButton()).not.toExist();
    });

    it('should propagate model/pristinity changes using callbacks', () => {
        const onModelChangeFn = jest.fn();
        const onPristinityChangeFn = jest.fn();
        const onValidityChangeFn = jest.fn();
        const onUserInputFn = jest.fn();

        const {changeInput, clearButton, inputField} = validations.buildFrom(<XlReactWidgetString
            metadata={{onUserInput: onUserInputFn}}
            onModelChange={onModelChangeFn}
            onPristinityChange={onPristinityChangeFn}
            onValidityChange={onValidityChangeFn}
        />);

        changeInput('new value');
        expect(inputField()).toHaveValue('new value');
        setTimeout(() => {
            expect(onModelChangeFn.mock.calls).toEqual([['', true], ['new value', false]]);
            expect(onPristinityChangeFn.mock.calls).toEqual([[true], [false]]);
            expect(onValidityChangeFn.mock.calls).toEqual([[true]]);
            expect(onUserInputFn.mock.calls).toEqual([['new value']]);
            expect(clearButton()).toExist();

            clearButton().simulate('click');
            expect(clearButton()).not.toExist();
            expect(inputField()).toHaveValue('');
            expect(onModelChangeFn.mock.calls).toEqual([['', true], ['new value', false], ['', false]]);
            expect(onPristinityChangeFn.mock.calls).toEqual([[true], [false]]);
            expect(onValidityChangeFn.mock.calls).toEqual([[true]]);
            expect(onUserInputFn.mock.calls).toEqual([['new value'], ['']]);
        }, 500);
    });

    it('should validate / propagate component validity', () => {
        const onValidityChangeFn = jest.fn();
        const {changeInput, clearButton, inputField} = validations
            .buildFrom(<XlReactWidgetString metadata={{required: true}} onValidityChange={onValidityChangeFn}/>);

        expect(inputField()).toExist();
        expect(inputField()).not.toHaveClassName('invalid');
        expect(onValidityChangeFn.mock.calls).toEqual([[false]]);

        changeInput('value');
        expect(inputField()).not.toHaveClassName('invalid');
        setTimeout(() => {
            expect(onValidityChangeFn.mock.calls).toEqual([[false], [true]]);
            clearButton().simulate('click');
            expect(inputField()).toHaveClassName('invalid');
            expect(onValidityChangeFn.mock.calls).toEqual([[false], [true], [false]]);
        }, 500);
    });

    it('should be able to show default error message if the field is emptied after typing something', () => {
        const onValidityChangeFn = jest.fn();
        const {changeInput, inputField, clearButton, errMsg} = validations.buildFrom(
            <XlReactWidgetString metadata={{required: true}} onValidityChange={onValidityChangeFn}/>
        );

        expect(inputField()).toExist();
        expect(inputField()).not.toHaveClassName('invalid');
        expect(onValidityChangeFn.mock.calls).toEqual([[false]]);

        changeInput('test value');

        setTimeout(() => {
            clearButton().simulate('click');
            expect(inputField()).toHaveClassName('invalid');
            expect(errMsg()).toExist();
            expect(errMsg()).toHaveText('This field is required');
        }, 500);
    });

    it('should be able to show custom error message for required fields', () => {
        const onValidityChangeFn = jest.fn();
        const {changeInput, clearButton, errMsg, inputField} = validations.buildFrom(<XlReactWidgetString
            metadata={{
                required: true,
                requiredText: 'Application name is required!'
            }}
            onValidityChange={onValidityChangeFn}
        />);

        expect(inputField()).toExist();
        expect(inputField()).not.toHaveClassName('invalid');
        expect(onValidityChangeFn.mock.calls).toEqual([[false]]);

        changeInput('test value');

        setTimeout(() => {
            clearButton().simulate('click');
            expect(inputField()).toHaveClassName('invalid');
            expect(errMsg()).toExist();
            expect(errMsg()).toHaveText('Application name is required!');
        }, 500);
    });

    it('should always show error received in props', () => {
        const onValidityChangeFn = jest.fn();
        const onModelChangeFn = jest.fn();
        const {inputField, wrapper} = validations.buildFrom(<XlReactWidgetString
            message={{show: true, text: 'this is an error', type: ERROR}}
            metadata={{required: true}}
            onModelChange={onModelChangeFn}
            onValidityChange={onValidityChangeFn}
        />);

        expect(inputField()).toExist();
        expect(wrapper).toHaveState({message: {show: true, text: 'this is an error', type: ERROR}, valid: false});
    });

    it('should always show warning received in props', () => {
        const onValidityChangeFn = jest.fn();
        const onModelChangeFn = jest.fn();
        const {inputField, wrapper} = validations.buildFrom(<XlReactWidgetString
            message={{show: true, text: 'this is a warning', type: WARNING}}
            onModelChange={onModelChangeFn}
            onValidityChange={onValidityChangeFn}
            options={{required: true}}
        />);

        expect(inputField()).toExist();
        expect(wrapper).toHaveState({message: {show: true, text: 'this is a warning', type: WARNING}, valid: true});
    });

    it('should show custom error message', () => {
        const onValidityChangeFn = jest.fn();
        const onModelChangeFn = jest.fn();
        const {changeInput, errMsg, inputField, wrapper} = validations.buildFrom(<XlReactWidgetString
            metadata={{required: true, requiredText: 'this is an error'}}
            onModelChange={onModelChangeFn}
            onValidityChange={onValidityChangeFn}
        />);

        expect(inputField()).toExist();
        expect(wrapper).toHaveState({message: {show: false, text: 'this is an error', type: ERROR}});

        changeInput('value');
        expect(errMsg()).not.toExist();

        changeInput('');
        inputField().simulate('blur');
        setTimeout(() => {
            expect(inputField()).toHaveClassName('invalid');
            expect(errMsg()).toExist();
            expect(errMsg()).toHaveText('this is an error');
        }, 500);
    });

    it('should have default value from options property', () => {
        const {inputField} = validations.buildFrom(
            <XlReactWidgetString metadata={{default: 'Options default value'}}/>
        );

        expect(inputField()).toHaveValue('Options default value');
    });

    it('should have default value from metadata property', () => {
        const {inputField} = validations.buildFrom(<XlReactWidgetString
            metadata={{default: 'Metadata default value'}}
        />);

        expect(inputField()).toHaveValue('Metadata default value');
    });

    it('should render small input if not valid prop "size" value is passed into component', () => {
        const {inputField} = validations.buildFrom(<XlReactWidgetString
            metadata={{size: 'notValidPropSize'}}
        />);

        expect(inputField()).toExist();
    });

    it('should be small size when "size" attribute is set to "small"', () => {
        const {inputField} = validations.buildFrom(<XlReactWidgetString
            metadata={{size: 'SMALL'}}
        />);
        expect(inputField()).toExist();
    });

    it('should be medium size when "size" attribute is set to "medium"', () => {
        const {wrapper} = validations.buildFrom(<XlReactWidgetString
            metadata={{size: 'MEDIUM'}}
        />);

        expect(wrapper.find('.xl-react-components-input-full div.xl-react-widget-medium-sized-string')).toExist();
    });

    it('should be large size when "size" attribute is set to "large"', () => {
        const {wrapper} = validations.buildFrom(<XlReactWidgetString
            metadata={{size: 'LARGE'}}
        />);

        expect(wrapper.find('.xl-react-components-input-full div.xl-react-widget-large-sized-string')).toExist();
    });

    it('should invoke onKeyPress if is defined', () => {
        const onModelChangeFn = jest.fn();
        const onKeyPressFn = jest.fn();
        const {changeInput, inputField} = validations.buildFrom(<XlReactWidgetString
            metadata={{label: 'My Label'}}
            onKeyPress={onKeyPressFn}
            onModelChange={onModelChangeFn}
        />);

        changeInput('value');
        inputField().simulate('keyPress', {key: 'Enter', keyCode: 13, which: 13});
        expect(onKeyPressFn).toBeCalled();
    });

    it('should not invoke onKeyPress if is not defined', () => {
        const onModelChangeFn = jest.fn();
        const onKeyPressFn = jest.fn();
        const {changeInput, inputField} = validations.buildFrom(<XlReactWidgetString
            metadata={{label: 'My Label'}}
            onModelChange={onModelChangeFn}
        />);

        changeInput('value');
        inputField().simulate('keyPress', {key: 'Enter', keyCode: 13, which: 13});
        expect(onKeyPressFn).not.toBeCalled();
    });
});
